@@ -1,130 +1,19 @@
 //! This is a debugging crate that provides better error messages for [`axum`] framework.
 //!
-//! While using [`axum`], you can get long error messages for simple mistakes. For example:
+//! **Note:** this crate is deprecated. Use [axum-macros] instead.
 //!
-//! ```rust,compile_fail
-//! use axum::{routing::get, Router};
+//! [axum-macros]: https://crates.io/crates/axum-macros
 //!
-//! #[tokio::main]
-//! async fn main() {
-//!     let app = Router::new().route("/", get(handler));
-//!
-//!     axum::Server::bind(&"0.0.0.0:3000".parse().unwrap())
-//!         .serve(app.into_make_service())
-//!         .await
-//!         .unwrap();
-//! }
-//!
-//! fn handler() -> &'static str {
-//!     "Hello, world"
-//! }
-//! ```
-//!
-//! You will get a long error message about function not implementing [`Handler`] trait. But why
-//! this function does not implement it? To figure it out [`debug_handler`] macro can be used.
-//!
-//! ```rust,compile_fail
-//! # use axum::{routing::get, Router};
-//! # use axum_debug::debug_handler;
-//! #
-//! # #[tokio::main]
-//! # async fn main() {
-//! #     let app = Router::new().route("/", get(handler));
-//! #
-//! #     axum::Server::bind(&"0.0.0.0:3000".parse().unwrap())
-//! #         .serve(app.into_make_service())
-//! #         .await
-//! #         .unwrap();
-//! # }
-//! #
-//! #[debug_handler]
-//! fn handler() -> &'static str {
-//!     "Hello, world"
-//! }
-//! ```
-//!
-//! ```text
-//! error: handlers must be async functions
-//!   --> main.rs:xx:1
-//!    |
-//! xx | fn handler() -> &'static str {
-//!    | ^^
-//! ```
-//!
-//! As the error message says, handler function needs to be async.
-//!
-//! ```rust,compile_fail
-//! use axum::{routing::get, Router};
-//! use axum_debug::debug_handler;
-//!
-//! #[tokio::main]
-//! async fn main() {
-//!     let app = Router::new().route("/", get(handler));
-//!
-//!     axum::Server::bind(&"0.0.0.0:3000".parse().unwrap())
-//!         .serve(app.into_make_service())
-//!         .await
-//!         .unwrap();
-//! }
-//!
-//! #[debug_handler]
-//! async fn handler() -> &'static str {
-//!     "Hello, world"
-//! }
-//! ```
-//!
-//! # Performance
-//!
-//! Macros in this crate have no effect when using release profile. (eg. `cargo build --release`)
-//!
-//! [`axum`]: https://docs.rs/axum/0.3
-//! [`Handler`]: https://docs.rs/axum/0.3/axum/handler/trait.Handler.html
-//! [`debug_handler`]: macro@debug_handler
-
-#![warn(
-    clippy::all,
-    clippy::dbg_macro,
-    clippy::todo,
-    clippy::empty_enum,
-    clippy::enum_glob_use,
-    clippy::mem_forget,
-    clippy::unused_self,
-    clippy::filter_map_next,
-    clippy::needless_continue,
-    clippy::needless_borrow,
-    clippy::match_wildcard_for_single_variants,
-    clippy::if_let_mutex,
-    clippy::mismatched_target_os,
-    clippy::await_holding_lock,
-    clippy::match_on_vec_items,
-    clippy::imprecise_flops,
-    clippy::suboptimal_flops,
-    clippy::lossy_float_literal,
-    clippy::rest_pat_in_fully_bound_structs,
-    clippy::fn_params_excessive_bools,
-    clippy::exit,
-    clippy::inefficient_to_string,
-    clippy::linkedlist,
-    clippy::macro_use_imports,
-    clippy::option_option,
-    clippy::verbose_file_reads,
-    clippy::unnested_or_patterns,
-    clippy::str_to_string,
-    rust_2018_idioms,
-    future_incompatible,
-    nonstandard_style,
-    missing_debug_implementations,
-    missing_docs
-)]
-#![deny(unreachable_pub, private_in_public)]
-#![allow(elided_lifetimes_in_paths, clippy::type_complexity)]
-#![forbid(unsafe_code)]
-#![cfg_attr(docsrs, feature(doc_cfg))]
-#![cfg_attr(test, allow(clippy::float_cmp))]
+//! [`axum`]: https://docs.rs/axum/latest
 
 use proc_macro::TokenStream;
 
 /// Generates better error messages when applied to a handler function.
+///
+/// Note this crate is deprecated. Use [axum-macros] instead.
+///
+/// [axum-macros]: https://crates.io/crates/axum-macros
+#[deprecated(since = "0.3.3", note = "Use the axum-macros crate instead")]
 #[proc_macro_attribute]
 pub fn debug_handler(_attr: TokenStream, input: TokenStream) -> TokenStream {
     #[cfg(not(debug_assertions))]
@@ -138,7 +27,7 @@ pub fn debug_handler(_attr: TokenStream, input: TokenStream) -> TokenStream {
 mod debug_handler {
     use proc_macro2::TokenStream;
     use quote::{format_ident, quote, quote_spanned};
-    use syn::{parse::Parse, spanned::Spanned, FnArg, ItemFn};
+    use syn::{parse::Parse, spanned::Spanned, FnArg, ItemFn, Token, Type};
 
     pub(crate) fn expand(
         attr: proc_macro::TokenStream,
@@ -151,12 +40,13 @@ mod debug_handler {
     }
 
     pub(crate) fn try_expand(attr: TokenStream, input: TokenStream) -> syn::Result<TokenStream> {
-        syn::parse2::<Attrs>(attr)?;
+        let attr = syn::parse2::<Attrs>(attr)?;
         let item_fn = syn::parse2::<ItemFn>(input.clone())?;
 
         check_extractor_count(&item_fn)?;
 
-        let check_inputs_impls_from_request = check_inputs_impls_from_request(&item_fn);
+        let check_inputs_impls_from_request =
+            check_inputs_impls_from_request(&item_fn, &attr.body_ty);
         let check_output_impls_into_response = check_output_impls_into_response(&item_fn);
         let check_future_send = check_future_send(&item_fn);
 
@@ -170,11 +60,29 @@ mod debug_handler {
         Ok(tokens)
     }
 
-    struct Attrs;
+    struct Attrs {
+        body_ty: Type,
+    }
 
     impl Parse for Attrs {
-        fn parse(_input: syn::parse::ParseStream) -> syn::Result<Self> {
-            Ok(Self)
+        fn parse(input: syn::parse::ParseStream) -> syn::Result<Self> {
+            let mut body_ty = None;
+
+            while !input.is_empty() {
+                let ident = input.parse::<syn::Ident>()?;
+                if ident == "body" {
+                    input.parse::<Token![=]>()?;
+                    body_ty = Some(input.parse()?);
+                } else {
+                    return Err(syn::Error::new_spanned(ident, "unknown argument"));
+                }
+
+                let _ = input.parse::<Token![,]>();
+            }
+
+            let body_ty = body_ty.unwrap_or_else(|| syn::parse_quote!(axum::body::Body));
+
+            Ok(Self { body_ty })
         }
     }
 
@@ -193,7 +101,7 @@ mod debug_handler {
         }
     }
 
-    fn check_inputs_impls_from_request(item_fn: &ItemFn) -> TokenStream {
+    fn check_inputs_impls_from_request(item_fn: &ItemFn, body_ty: &Type) -> TokenStream {
         if !item_fn.sig.generics.params.is_empty() {
             return syn::Error::new_spanned(
                 &item_fn.sig.generics,
@@ -237,7 +145,7 @@ mod debug_handler {
                     #[allow(warnings)]
                     fn #name()
                     where
-                        #ty: ::axum::extract::FromRequest<::axum::body::Body> + Send,
+                        #ty: ::axum::extract::FromRequest<#body_ty> + Send,
                     {}
                 }
             })
@@ -405,18 +313,3 @@ mod debug_handler {
         None
     }
 }
-
-#[test]
-fn ui() {
-    #[rustversion::stable]
-    fn go() {
-        let t = trybuild::TestCases::new();
-        t.compile_fail("tests/fail/*.rs");
-        t.pass("tests/pass/*.rs");
-    }
-
-    #[rustversion::not(stable)]
-    fn go() {}
-
-    go();
-}
